package com.dzeio.openhealth.ui.weight

import android.Manifest
import android.content.SharedPreferences
import android.graphics.Paint
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.view.MenuProvider
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.charts.Entry
import com.dzeio.charts.axis.Line
import com.dzeio.charts.series.LineSerie
import com.dzeio.openhealth.BuildConfig
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.ItemAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentListWeightBinding
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.PermissionsUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import java.text.DateFormat
import java.util.Date
import java.util.Locale

@AndroidEntryPoint
class ListWeightFragment :
    BaseFragment<ListWeightViewModel, FragmentListWeightBinding>(ListWeightViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) ->
    FragmentListWeightBinding = FragmentListWeightBinding::inflate

    private lateinit var button: View
    private val activityResult = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) {
        if (it.containsValue(false)) {
            // TODO: Show a popup with choice to change it
            Toast.makeText(requireContext(), R.string.permission_declined, Toast.LENGTH_LONG).show()
            return@registerForActivityResult
        }

        button.callOnClick()
    }

    private val menuProvider = object : MenuProvider {
        override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {
            menu.findItem(R.id.action_add).isVisible = true
        }

        override fun onMenuItemSelected(item: MenuItem): Boolean {
            return when (item.itemId) {
                R.id.action_add -> {
                    findNavController().navigate(
                        ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                            WeightDialog.DialogTypes.ADD_WEIGHT.ordinal
                        )
                    )
                    true
                }

                else -> false
            }
        }
    }

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Menu
        requireActivity().addMenuProvider(menuProvider)

        if (viewModel.goalWeight.value != null) {
            binding.goalButton.setText(R.string.edit_goal)
        }

        binding.goalButton.setOnClickListener {
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                    WeightDialog.DialogTypes.EDIT_GOAL.ordinal
                )
            )
        }

        binding.bluetoothButton.setOnClickListener {
            val permissions = arrayOf(
                Manifest.permission.BLUETOOTH,
                Manifest.permission.BLUETOOTH_CONNECT,
                Manifest.permission.BLUETOOTH_SCAN,
                Manifest.permission.ACCESS_FINE_LOCATION
            )
            val hasPermission = PermissionsUtils.hasPermission(requireContext(), permissions)
            if (!hasPermission) {
                button = binding.bluetoothButton
                activityResult.launch(permissions)
                return@setOnClickListener
            }
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToScanScalesDialog()
            )
        }

        val adapter = ItemAdapter<Weight>().apply {
            onItemClick = {
                findNavController().navigate(
                    ListWeightFragmentDirections.actionNavListWeightToNavEditWeight(
                        it.value.id
                    )
                )
            }
        }

        binding.list.apply {
            val manager = LinearLayoutManager(requireContext())
            layoutManager = manager
            this.adapter = adapter
        }

        viewModel.goalWeight.observe(viewLifecycleOwner) {
            binding.chart.yAxis.apply {
                clearLines()
                if (it != null) {
                    addLine(
                        it,
                        Line(true, Paint(linePaint).apply { strokeWidth = 4f })
                    )
                }
                binding.chart.refresh()
            }
        }

        viewModel.weights.observe(viewLifecycleOwner) { list ->
            if (list != null) {
                val unit = viewModel.massUnit.value ?: Units.Mass.KILOGRAM
                adapter.set(
                    list.map {
                        ItemAdapter.Item(
                            it,
                            getString(
                                unit.unit,
                                unit.formatToString(it.weight)
                            ),
                            it.formatTimestamp(),
                            icon = R.drawable.ic_outline_edit_24
                        )
                    }.reversed()
                )
                updateGraph(list)
            }
        }

        val chart = binding.chart

        val serie = LineSerie(chart).apply {
            linePaint.color = MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            )
            textPaint.color = MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnPrimary
            )
        }

        chart.apply {
            series = arrayListOf(serie)

            yAxis.apply {
                textLabel.color = MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
                linePaint.color = MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )

                onValueFormat = { value -> "${value.toInt()}" }
            }

            xAxis.apply {
                // 7 day history
                textPaint.color = MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
                textPaint.textSize = 32f
                onValueFormat = onValueFormat@{
                    val formatter = DateFormat.getDateTimeInstance(
                        DateFormat.SHORT,
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                    return@onValueFormat formatter.format(Date(it.toLong()))
                }
            }
        }

        // Debug button
        if (BuildConfig.DEBUG) {
            binding.debugRandomValues.visibility = View.VISIBLE
            binding.debugRandomValues.setOnClickListener {
                viewModel.generateRandomValues()
            }
            binding.debugRandomValues.setOnLongClickListener {
                viewModel.delete(viewModel.weights.value!!)
                return@setOnLongClickListener true
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        requireActivity().removeMenuProvider(menuProvider)
    }

    private fun updateGraph(list: List<Weight>) {
        val chart = binding.chart
        val serie = chart.series[0] as LineSerie

        val entries: ArrayList<Entry> = arrayListOf()

        list.forEach {
            entries.add(
                Entry(
                    it.timestamp.toDouble(),
                    it.weight
                )
            )
        }
        serie.entries = entries

        if (list.isEmpty()) {
            chart.xAxis.x = 0.0
        } else {
            chart.xAxis.x = list[0].timestamp.toDouble()
        }

        chart.refresh()
    }
}
